/*
 * Copyright 2004 Freescale Semiconductor.
 * (C) Copyright 2003,Motorola Inc.
 * Xianghua Xiao, (X.Xiao@motorola.com)
 *
 * (C) Copyright 2002 Scott McNutt <smcnutt@artesyncp.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <common.h>
#include <pci.h>
#include <asm/processor.h>
#include <asm/immap_85xx.h>
#include <ioports.h>
#include <spd.h>
#include <miiphy.h>

#if defined(CONFIG_DDR_ECC) && !defined(CONFIG_ECC_INIT_VIA_DDRCONTROLLER)
extern void ddr_enable_ecc(unsigned int dram_size);
#endif

extern long int spd_sdram(void);

void local_bus_init(void);
void sdram_init(void);
long int fixed_sdram(void);
void interrupt_init_pic(void);
#if 0
void pci_mpc85xx_init(struct pci_controller *hose);
void test_64bit_pci(void);
#endif

/*
 * I/O Port configuration table
 *
 * if conf is 1, then that port pin will be configured at boot time
 * according to the five values podr/pdir/ppar/psor/pdat for that entry
 */

const iop_conf_t iop_conf_tab[4][32] = {

    /* Port A configuration */
    {   /*            conf ppar psor pdir podr pdat */
	/* PA31 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 TxENB */
	/* PA30 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 TxClav   */
	/* PA29 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 TxSOC  */
	/* PA28 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 RxENB */
	/* PA27 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 RxSOC */
	/* PA26 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 RxClav */
	/* PA25 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[0] */
	/* PA24 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[1] */
	/* PA23 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[2] */
	/* PA22 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[3] */
	/* PA21 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[4] */
	/* PA20 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[5] */
	/* PA19 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[6] */
	/* PA18 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[7] */
	/* PA17 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[7] */
	/* PA16 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[6] */
	/* PA15 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[5] */
	/* PA14 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[4] */
	/* PA13 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[3] */
	/* PA12 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[2] */
	/* PA11 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[1] */
	/* PA10 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[0] */
	/* PA9  */ {   0,   1,   1,   1,   0,   0   }, /* FCC1 L1TXD */
	/* PA8  */ {   0,   1,   1,   0,   0,   0   }, /* FCC1 L1RXD */
	/* PA7  */ {   0,   0,   0,   1,   0,   0   }, /* PA7 */
	/* PA6  */ {   0,   1,   1,   1,   0,   0   }, /* TDM A1 L1RSYNC */
	/* PA5  */ {   0,   0,   0,   1,   0,   0   }, /* PA5 */
	/* PA4  */ {   0,   0,   0,   1,   0,   0   }, /* PA4 */
	/* PA3  */ {   0,   0,   0,   1,   0,   0   }, /* PA3 */
	/* PA2  */ {   0,   0,   0,   1,   0,   0   }, /* PA2 */
	/* PA1  */ {   0,   0,   0,   0,   0,   0   }, /* FREERUN */
	/* PA0  */ {   0,   0,   0,   1,   0,   0   }  /* PA0 */
    },

    /* Port B configuration */
    {   /*            conf ppar psor pdir podr pdat */
	/* PB31 */ {   0,   1,   0,   1,   0,   0   }, /* FCC2 MII TX_ER */
	/* PB30 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII RX_DV */
	/* PB29 */ {   0,   1,   1,   1,   0,   0   }, /* FCC2 MII TX_EN */
	/* PB28 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII RX_ER */
	/* PB27 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII COL */
	/* PB26 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII CRS */
	/* PB25 */ {   0,   1,   0,   1,   0,   0   }, /* FCC2 MII TxD[3] */
	/* PB24 */ {   0,   1,   0,   1,   0,   0   }, /* FCC2 MII TxD[2] */
	/* PB23 */ {   0,   1,   0,   1,   0,   0   }, /* FCC2 MII TxD[1] */
	/* PB22 */ {   0,   1,   0,   1,   0,   0   }, /* FCC2 MII TxD[0] */
	/* PB21 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII RxD[0] */
	/* PB20 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII RxD[1] */
	/* PB19 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII RxD[2] */
	/* PB18 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII RxD[3] */
	/* PB17 */ {   0,   1,   0,   0,   0,   0   }, /* FCC3:RX_DIV */
	/* PB16 */ {   0,   1,   0,   0,   0,   0   }, /* FCC3:RX_ERR */
	/* PB15 */ {   0,   1,   0,   1,   0,   0   }, /* FCC3:TX_ERR */
	/* PB14 */ {   0,   1,   0,   1,   0,   0   }, /* FCC3:TX_EN */
	/* PB13 */ {   0,   1,   0,   0,   0,   0   }, /* FCC3:COL */
	/* PB12 */ {   0,   1,   0,   0,   0,   0   }, /* FCC3:CRS */
	/* PB11 */ {   0,   1,   0,   0,   0,   0   }, /* FCC3:RXD */
	/* PB10 */ {   0,   1,   0,   0,   0,   0   }, /* FCC3:RXD */
	/* PB9  */ {   0,   1,   0,   0,   0,   0   }, /* FCC3:RXD */
	/* PB8  */ {   0,   1,   0,   0,   0,   0   }, /* FCC3:RXD */
	/* PB7  */ {   0,   1,   0,   1,   0,   0   }, /* FCC3:TXD */
	/* PB6  */ {   0,   1,   0,   1,   0,   0   }, /* FCC3:TXD */
	/* PB5  */ {   0,   1,   0,   1,   0,   0   }, /* FCC3:TXD */
	/* PB4  */ {   0,   1,   0,   1,   0,   0   }, /* FCC3:TXD */
	/* PB3  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PB2  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PB1  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PB0  */ {   0,   0,   0,   0,   0,   0   }  /* pin doesn't exist */
    },

    /* Port C */
    {   /*            conf ppar psor pdir podr pdat */
	/* PC31 */ {   0,   0,   0,   1,   0,   0   }, /* PC31 */
	/* PC30 */ {   0,   0,   0,   1,   0,   0   }, /* PC30 */
	/* PC29 */ {   0,   1,   1,   0,   0,   0   }, /* SCC1 EN *CLSN */
	/* PC28 */ {   0,   0,   0,   1,   0,   0   }, /* PC28 */
	/* PC27 */ {   0,   0,   0,   1,   0,   0   }, /* UART Clock in */
	/* PC26 */ {   0,   0,   0,   1,   0,   0   }, /* PC26 */
	/* PC25 */ {   0,   0,   0,   1,   0,   0   }, /* PC25 */
	/* PC24 */ {   0,   0,   0,   1,   0,   0   }, /* PC24 */
	/* PC23 */ {   0,   1,   0,   1,   0,   0   }, /* ATMTFCLK */
	/* PC22 */ {   0,   1,   0,   0,   0,   0   }, /* ATMRFCLK */
	/* PC21 */ {   0,   1,   0,   0,   0,   0   }, /* SCC1 EN RXCLK */
	/* PC20 */ {   0,   1,   0,   0,   0,   0   }, /* SCC1 EN TXCLK */
	/* PC19 */ {   0,   1,   0,   0,   0,   0   }, /* FCC2 MII RX_CLK CLK13 */
	/* PC18 */ {   0,   1,   0,   0,   0,   0   }, /* FCC Tx Clock (CLK14) */
	/* PC17 */ {   0,   0,   0,   1,   0,   0   }, /* PC17 */
	/* PC16 */ {   0,   1,   0,   0,   0,   0   }, /* FCC Tx Clock (CLK16) */
	/* PC15 */ {   0,   1,   0,   0,   0,   0   }, /* PC15 */
	/* PC14 */ {   0,   1,   0,   0,   0,   0   }, /* SCC1 EN *CD */
	/* PC13 */ {   0,   0,   0,   1,   0,   0   }, /* PC13 */
	/* PC12 */ {   0,   1,   0,   1,   0,   0   }, /* PC12 */
	/* PC11 */ {   0,   0,   0,   1,   0,   0   }, /* LXT971 transmit control */
	/* PC10 */ {   0,   0,   0,   1,   0,   0   }, /* FETHMDC */
	/* PC9  */ {   0,   0,   0,   0,   0,   0   }, /* FETHMDIO */
	/* PC8  */ {   0,   0,   0,   1,   0,   0   }, /* PC8 */
	/* PC7  */ {   0,   0,   0,   1,   0,   0   }, /* PC7 */
	/* PC6  */ {   0,   0,   0,   1,   0,   0   }, /* PC6 */
	/* PC5  */ {   0,   0,   0,   1,   0,   0   }, /* PC5 */
	/* PC4  */ {   0,   0,   0,   1,   0,   0   }, /* PC4 */
	/* PC3  */ {   0,   0,   0,   1,   0,   0   }, /* PC3 */
	/* PC2  */ {   0,   0,   0,   1,   0,   1   }, /* ENET FDE */
	/* PC1  */ {   0,   0,   0,   1,   0,   0   }, /* ENET DSQE */
	/* PC0  */ {   0,   0,   0,   1,   0,   0   }, /* ENET LBK */
    },

    /* Port D */
    {   /*            conf ppar psor pdir podr pdat */
	/* PD31 */ {   1,   1,   0,   0,   0,   0   }, /* SCC1 EN RxD */
	/* PD30 */ {   1,   1,   1,   1,   0,   0   }, /* SCC1 EN TxD */
	/* PD29 */ {   0,   1,   0,   1,   0,   0   }, /* SCC1 EN TENA */
	/* PD28 */ {   0,   1,   0,   0,   0,   0   }, /* PD28 */
	/* PD27 */ {   0,   1,   1,   1,   0,   0   }, /* PD27 */
	/* PD26 */ {   0,   0,   0,   1,   0,   0   }, /* PD26 */
	/* PD25 */ {   0,   0,   0,   1,   0,   0   }, /* PD25 */
	/* PD24 */ {   0,   0,   0,   1,   0,   0   }, /* PD24 */
	/* PD23 */ {   0,   0,   0,   1,   0,   0   }, /* PD23 */
	/* PD22 */ {   0,   0,   0,   1,   0,   0   }, /* PD22 */
	/* PD21 */ {   0,   0,   0,   1,   0,   0   }, /* PD21 */
	/* PD20 */ {   0,   0,   0,   1,   0,   0   }, /* PD20 */
	/* PD19 */ {   0,   0,   0,   1,   0,   0   }, /* PD19 */
	/* PD18 */ {   0,   0,   0,   1,   0,   0   }, /* PD18 */
	/* PD17 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXPRTY */
	/* PD16 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXPRTY */
	/* PD15 */ {   0,   1,   1,   0,   1,   0   }, /* I2C SDA */
	/* PD14 */ {   0,   0,   0,   1,   0,   0   }, /* LED */
	/* PD13 */ {   0,   0,   0,   0,   0,   0   }, /* PD13 */
	/* PD12 */ {   0,   0,   0,   0,   0,   0   }, /* PD12 */
	/* PD11 */ {   0,   0,   0,   0,   0,   0   }, /* PD11 */
	/* PD10 */ {   0,   0,   0,   0,   0,   0   }, /* PD10 */
	/* PD9  */ {   0,   1,   0,   1,   0,   0   }, /* SMC1 TXD */
	/* PD8  */ {   0,   1,   0,   0,   0,   0   }, /* SMC1 RXD */
	/* PD7  */ {   0,   0,   0,   1,   0,   1   }, /* PD7 */
	/* PD6  */ {   0,   0,   0,   1,   0,   1   }, /* PD6 */
	/* PD5  */ {   0,   0,   0,   1,   0,   1   }, /* PD5 */
	/* PD4  */ {   0,   0,   0,   1,   0,   1   }, /* PD4 */
	/* PD3  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PD2  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PD1  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PD0  */ {   0,   0,   0,   0,   0,   0   }  /* pin doesn't exist */
    }
};


/*
 * EP8560M Board Control & Status Registers
 */
typedef struct bcsr_ {
	volatile unsigned char bcsr0;
	volatile unsigned char bcsr1;
	volatile unsigned char bcsr2;
	volatile unsigned char bcsr3;
	volatile unsigned char bcsr4;
	volatile unsigned char bcsr5;
	volatile unsigned char bcsr6;
	volatile unsigned char bcsr7;
	volatile unsigned char bcsr8;
	volatile unsigned char bcsr9;
	volatile unsigned char bcsr10;
	volatile unsigned char bcsr11;
	volatile unsigned char bcsr12;
	volatile unsigned char bcsr13;
	volatile unsigned char bcsr14;
	volatile unsigned char bcsr15;
} bcsr_t;


int board_early_init_f (void)
{
	volatile immap_t *immap = (immap_t *)CFG_IMMR;
	volatile bcsr_t *bcsr = (bcsr_t *) CFG_BCSR;
	char temp;

	/* Not enabled in cpu_init.c if CONFIG_MPC85xx */
	/* Board uses CS2 for BCSR registers */
	immap->im_lbc.or2 = CFG_OR2_PRELIM;
	immap->im_lbc.br2 = CFG_BR2_PRELIM;

	/* enable SRAM, clear FLASH and SRAM write protect */
	temp  = bcsr->bcsr11;
	temp |= BCSR11_SRAM_EN | BCSR11_SRAM_WP | BCSR11_FLASH_WP;
	bcsr->bcsr11 = temp;

#ifdef CONFIG_CLK_OUT
	/* enable ccb clock at CLK_OUT */
	immap->im_gur.clkocr = 0x80000000;
#endif

#ifdef CONFIG_PCI
	/* unmask PCI interrupts */
	temp  = bcsr->bcsr15;
	temp &= ~(BCSR15_PCI_INTD_MASK | BCSR15_PCI_INTC_MASK |
			  BCSR15_PCI_INTB_MASK |BCSR15_PCI_INTA_MASK);
	bcsr->bcsr15 = temp;
#endif

#if defined(CONFIG_TSEC_ENET) /* avoid compile warnings for now */
	/* clear COMA bit, both ports */
	temp  = bcsr->bcsr13;
	temp &= ~(BCSR13_GETH1_COMA | BCSR13_GETH2_COMA);
	bcsr->bcsr13 = temp;

	/* reset Gigabit Ethernet port, both ports */
	temp  = bcsr->bcsr13;
	temp &= ~(BCSR13_GETH1_RESET | BCSR13_GETH2_RESET);
	bcsr->bcsr13 = temp;
	udelay(1000);
	temp |= (BCSR13_GETH1_RESET | BCSR13_GETH2_RESET);
	bcsr->bcsr13 = temp;
	udelay(1000);
#endif

	interrupt_init_pic();

    return 0;
}

int board_early_init_r (void)
{
    return 0;
}

void interrupt_init_pic (void)
{
	volatile immap_t *immap = (immap_t *)CFG_IMMR;
	int i, temp;

	/* Reset the PIC */
	immap->im_pic.gcr = MPC85xx_PICGCR_RST;
	while (immap->im_pic.gcr & MPC85xx_PICGCR_RST);

	/* Set the vector, priority, and polarity */
#ifdef EIVPR_IRQ0
	immap->im_pic.eivpr0 |= (EIVPR_IRQ0_POLARITY | (EIVPR_IRQ0_PRIORITY<<15) | EIVPR_IRQ0_VECTOR);
	immap->im_pic.eidr0   = 0x1;	/*always processor 0*/
#endif

#ifdef EIVPR_IRQ1
	immap->im_pic.eivpr1 |= (EIVPR_IRQ1_POLARITY | (EIVPR_IRQ1_PRIORITY<<15) | EIVPR_IRQ1_VECTOR);
	immap->im_pic.eidr1   = 0x1;	/*always processor 0*/
#endif

#ifdef EIVPR_IRQ2
	immap->im_pic.eivpr2 |= (EIVPR_IRQ2_POLARITY | (EIVPR_IRQ2_PRIORITY<<15) | EIVPR_IRQ2_VECTOR);
	immap->im_pic.eidr2   = 0x1;	/*always processor 0*/
#endif

#ifdef EIVPR_IRQ3
	immap->im_pic.eivpr3 |= (EIVPR_IRQ3_POLARITY | (EIVPR_IRQ3_PRIORITY<<15) | EIVPR_IRQ3_VECTOR);
	immap->im_pic.eidr3   = 0x1;	/*always processor 0*/
#endif

#ifdef EIVPR_IRQ4
	immap->im_pic.eivpr4 |= (EIVPR_IRQ4_POLARITY | (EIVPR_IRQ4_PRIORITY<<15) | EIVPR_IRQ4_VECTOR);
	immap->im_pic.eidr4   = 0x1;	/*always processor 0*/
#endif

#ifdef EIVPR_IRQ5
	immap->im_pic.eivpr5 |= (EIVPR_IRQ5_POLARITY | (EIVPR_IRQ5_PRIORITY<<15) | EIVPR_IRQ5_VECTOR);
	immap->im_pic.eidr5   = 0x1;	/*always processor 0*/
#endif

#ifdef EIVPR_IRQ6
	immap->im_pic.eivpr6 |= (EIVPR_IRQ6_POLARITY | (EIVPR_IRQ6_PRIORITY<<15) | EIVPR_IRQ6_VECTOR);
	immap->im_pic.eidr6   = 0x1;	/*always processor 0*/
#endif

#ifdef EIVPR_IRQ7
	immap->im_pic.eivpr7 |= (EIVPR_IRQ7_POLARITY | (EIVPR_IRQ7_PRIORITY<<15) | EIVPR_IRQ7_VECTOR);
	immap->im_pic.eidr7   = 0x1	/*always processor 0*/;
#endif

#ifdef EIVPR_IRQ8
	immap->im_pic.eivpr8 |= (EIVPR_IRQ8_POLARITY | (EIVPR_IRQ8_PRIORITY<<15) | EIVPR_IRQ8_VECTOR);
	immap->im_pic.eidr8   = 0x1;	/*always processor 0*/
#endif


	/* Clear the CTPR */
	immap->im_pic.ctpr0   = 0;

	/* Program the PIC to mixed mode by setting */
	immap->im_pic.gcr = MPC85xx_PICGCR_M;

	/* Clear the MSK bit in the vector/priority registers to be used */
	immap->im_pic.eivpr1 &= ~EIVPR_MSK;
	immap->im_pic.eivpr2 &= ~EIVPR_MSK;
	immap->im_pic.eivpr3 &= ~EIVPR_MSK;
	immap->im_pic.eivpr4 &= ~EIVPR_MSK;
	immap->im_pic.eivpr5 &= ~EIVPR_MSK;
	immap->im_pic.eivpr6 &= ~EIVPR_MSK;

	/* Perform a software loop to clear all pending interrupts */
	for(i=0; i<MAX_NUM_EXT_INTS; i++)
	{
		temp = immap->im_pic.iack;
		immap->im_pic.eoi = 0;
	}

	/* Set the processor CTPR value to the desired value */
	immap->im_pic.ctpr0   = CTPR_TASK_PRIORITY;

}

void reset_phy (void)
{
	return;
}

int get_sys_clk (void)
{
	volatile bcsr_t *bcsr = (bcsr_t *) CFG_BCSR;

    return ((bcsr->bcsr4<<8) | bcsr->bcsr5) * 10000;
}

int get_pci_mode (void)
{
	volatile bcsr_t *bcsr = (bcsr_t *) CFG_BCSR;

    return (bcsr->bcsr9 & BCSR9_MONARCH)>>6;
}

int get_pci_x (void)
{
	volatile bcsr_t *bcsr = (bcsr_t *) CFG_BCSR;
	char temp;

	temp  = (bcsr->bcsr9 & BCSR9_M66EN)>>5;
	temp |= (bcsr->bcsr9 & BCSR9_PCIXCAP)>>4;

    return temp;
}

int get_pci_bus_width (void)
{
	volatile immap_t *immap = (immap_t *)CFG_IMMR;

    return (immap->im_gur.pordevsr & 0x00010000)>>16;
}

int get_pci_arbiter (void)
{
	volatile immap_t *immap = (immap_t *)CFG_IMMR;

    return (immap->im_gur.pordevsr & 0x00020000)>>17;
}

int get_board_type (void)
{
	volatile bcsr_t *bcsr = (bcsr_t *) CFG_BCSR;

    return (bcsr->bcsr2 & BCSR2_CPU_TYPE)>>3;
}

int checkboard (void)
{
	const char *board_type = NULL;
	const char *pci_mode = NULL;
	const char *pci_bus_width = NULL;
	const char *pci_arbiter = NULL;
	const char *pci_x = NULL;

	switch(get_board_type())
	{
		case 2:
		case 3:
		case 4:
		case 5:
			board_type = "EP8541M";
			break;
		case 6:
		case 7:
		case 8:
		case 9:
			board_type = "EP8555M";
			break;
		case 10:
		case 11:
			board_type = "EP8560M";
			break;
		default:
			board_type = "unknown";
			break;

	}
	printf ("Board: %s\n", board_type);

#if defined (CONFIG_PCI) && defined (CONFIG_SYS_CLK_FREQ)
	printf("    PCI: 64 bit, %d MHz (compiled)\n",
	       CONFIG_SYS_CLK_FREQ / 1000000);
#elif defined (CONFIG_PCI)

	switch(get_pci_x())
	{
		case 3:
		case 7:
			pci_x = "PCI";
			break;
		default:
			pci_x = "PCI-X";
			break;
	}

	switch(get_pci_mode())
	{
		case 0:
			pci_mode = "monarch";
			break;
		case 1:
			pci_mode = "non-monarch";
			break;
	}

	switch(get_pci_bus_width())
	{
		case 0:
			pci_bus_width = "64 bit";
			break;
		case 1:
			pci_bus_width = "32 bit";
			break;
	}

	switch(get_pci_arbiter())
	{
		case 0:
			pci_arbiter = "local arbiter disabled";
			break;
		case 1:
			pci_arbiter = "local arbiter enabled";
			break;
	}

	printf("    %s: %s, %d MHz, %s, %s\n", pci_x, pci_bus_width, get_sys_clk()/1000000, pci_mode, pci_arbiter);
#else
	printf("    PCI: disabled\n");
#endif

	/*
	 * Initialize local bus.
	 */
	local_bus_init();

	return 0;
}


long int
initdram(int board_type)
{
	long dram_size = 0;
	extern long spd_sdram (void);
	volatile immap_t *immap = (immap_t *)CFG_IMMR;

	puts("Initializing\n");

#if defined(CONFIG_DDR_DLL)
	{
	    volatile ccsr_gur_t *gur= &immap->im_gur;
	    uint temp_ddrdll = 0;

	    /*
	     * Work around to stabilize DDR DLL
	     */
	    temp_ddrdll = gur->ddrdllcr;
	    gur->ddrdllcr = ((temp_ddrdll & 0xff) << 16) | 0x80000000;
	    asm("sync;isync;msync");
	}
#endif

	dram_size = fixed_sdram ();

#if defined(CONFIG_DDR_ECC)
	/*
	 * Initialize and enable DDR ECC.
	 */
	ddr_enable_ecc(dram_size);
#endif

#if defined(CONFIG_DDR_ECC_DISCRETE)
	puts("    DDR (ECC): ");
#else
	puts("    DDR: ");
#endif
	return dram_size;
}


/*
 * Initialize Local Bus
 */

void
local_bus_init(void)
{
	volatile immap_t *immap = (immap_t *)CFG_IMMR;
	volatile ccsr_gur_t *gur = &immap->im_gur;
	volatile ccsr_lbc_t *lbc = &immap->im_lbc;

	uint clkdiv;
	uint lbc_hz;
	sys_info_t sysinfo;

	/*
	 * Errata LBC11.
	 * Fix Local Bus clock glitch when DLL is enabled.
	 *
	 * If localbus freq is < 66Mhz, DLL bypass mode must be used.
	 * If localbus freq is > 133Mhz, DLL can be safely enabled.
	 * Between 66 and 133, the DLL is enabled with an override workaround.
	 */

	get_sys_info(&sysinfo);
	clkdiv = lbc->lcrr & 0x0f;
	lbc_hz = sysinfo.freqSystemBus / 1000000 / clkdiv;

	if (lbc_hz < 66) {
		lbc->lcrr = CFG_LBC_LCRR | 0x80000000;	/* DLL Bypass */

	} else if (lbc_hz >= 133) {
		lbc->lcrr = CFG_LBC_LCRR & (~0x80000000); /* DLL Enabled */

	} else {
		/*
		 * On REV1 boards, need to change CLKDIV before enable DLL.
		 * Default CLKDIV is 8, change it to 4 temporarily.
		 */
		uint pvr = get_pvr();
		uint temp_lbcdll = 0;

		if (pvr == PVR_85xx_REV1) {
			/* FIXME: Justify the high bit here. */
			lbc->lcrr = 0x10000004;
		}

		lbc->lcrr = CFG_LBC_LCRR & (~0x80000000);/* DLL Enabled */
		udelay(200);

		/*
		 * Sample LBC DLL ctrl reg, upshift it to set the
		 * override bits.
		 */
		temp_lbcdll = gur->lbcdllcr;
		gur->lbcdllcr = (((temp_lbcdll & 0xff) << 16) | 0x80000000);
		asm("sync;isync;msync");
	}
}


#if defined(CFG_DRAM_TEST)
int testdram (void)
{
	uint *pstart = (uint *) CFG_MEMTEST_START;
	uint *pend = (uint *) CFG_MEMTEST_END;
	uint *p;

	printf("SDRAM test phase 1:\n");
	for (p = pstart; p < pend; p++)
		*p = 0xaaaaaaaa;

	for (p = pstart; p < pend; p++) {
		if (*p != 0xaaaaaaaa) {
			printf ("SDRAM test fails at: %08x\n", (uint) p);
			return 1;
		}
	}

	printf("SDRAM test phase 2:\n");
	for (p = pstart; p < pend; p++)
		*p = 0x55555555;

	for (p = pstart; p < pend; p++) {
		if (*p != 0x55555555) {
			printf ("SDRAM test fails at: %08x\n", (uint) p);
			return 1;
		}
	}

	printf("SDRAM test passed.\n");
	return 0;
}
#endif


/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 ************************************************************************/
long int fixed_sdram (void)
{
  #ifndef CFG_RAMBOOT
	volatile immap_t *immap = (immap_t *)CFG_IMMR;
	volatile ccsr_ddr_t *ddr= &immap->im_ddr;
	int i;

	ddr->cs0_bnds       = CFG_DDR_CS0_BNDS;
	ddr->cs0_config     = CFG_DDR_CS0_CONFIG;
	ddr->timing_cfg_1   = CFG_DDR_TIMING_1;
	ddr->timing_cfg_2   = CFG_DDR_TIMING_2;
	ddr->sdram_mode     = CFG_DDR_MODE;
	ddr->sdram_interval = CFG_DDR_INTERVAL;
    #if defined (CONFIG_DDR_ECC_DISCRETE)
	ddr->err_int_en     = 0x00000000;
	ddr->err_disable    = 0x00000009;
	ddr->err_sbe        = 0x00ff0000;
	ddr->err_detect     = 0x8000000D;
    #endif
	asm("sync;isync;msync");
	udelay(500);
    #if defined (CONFIG_DDR_ECC_DISCRETE)
	/* Enable ECC checking */
	ddr->sdram_cfg = (CFG_DDR_CONTROL | 0x20000000);
    #else
	ddr->sdram_cfg = CFG_DDR_CONTROL;
    #endif
	asm("sync; isync; msync");
	udelay(500);
  #endif
	return CFG_SDRAM_SIZE * 1024 * 1024;
}


#if defined(CONFIG_PCI)
/*
 * Initialize PCI Devices, report devices found.
 */

#ifndef CONFIG_PCI_PNP
static struct pci_config_table pci_ep85xxm_config_table[] = {
    { PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID,
      PCI_IDSEL_NUMBER, PCI_ANY_ID,
      pci_cfgfunc_config_device, { PCI_ENET0_IOADDR,
				   PCI_ENET0_MEMADDR,
				   PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER
      } },
    { }
};
#endif


static struct pci_controller hose = {
#ifndef CONFIG_PCI_PNP
	config_table: pci_ep85xxm_config_table,
#endif
};

#endif	/* CONFIG_PCI */


void
pci_init_board(void)
{
#ifdef CONFIG_PCI
#if 0
	extern void pci_mpc85xx_init(struct pci_controller *hose);
#endif

	pci_mpc85xx_init(&hose);
#endif /* CONFIG_PCI */
}

#if 0
void
pci_mpc85xx_init(struct pci_controller *hose)
{
	volatile immap_t    *immap = (immap_t *)CFG_CCSRBAR;
	volatile ccsr_pcix_t *pcix = &immap->im_pcix;

	u16 reg16;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	pci_set_region(hose->regions + 0,
		       CFG_PCI1_MEM_BASE,
		       CFG_PCI1_MEM_PHYS,
		       CFG_PCI1_MEM_SIZE,
		       PCI_REGION_MEM);

	pci_set_region(hose->regions + 1,
		       CFG_PCI1_IO_BASE,
		       CFG_PCI1_IO_PHYS,
		       CFG_PCI1_IO_SIZE,
		       PCI_REGION_IO);

	hose->region_count = 2;

	pci_setup_indirect(hose,
			   (CFG_IMMR+0x8000),
			   (CFG_IMMR+0x8004));

	if(!get_pci_mode())
	{
#if 0
		pci_read_config_word (PCI_BDF(0,0,0), PCI_COMMAND, &reg16);
		reg16 |= PCI_COMMAND_SERR | PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY;
		pci_write_config_word(PCI_BDF(0,0,0), PCI_COMMAND, reg16);
#endif
		immap->im_pcix.cfg_addr = 0x80000004;
		immap->im_pcix.cfg_data = 0x06000080;
	}

#if 0
	pci_write_config_word(PCI_BDF(0,0,0), 0x44, 0x0020);

	pci_read_config_word (PCI_BDF(0,0,0), PCI_DEVICE_ID, &reg16);
	printf("PCI_DEVICE_ID = %x\n",(int)reg16);
	pci_read_config_word (PCI_BDF(0,0,0), PCI_COMMAND, &reg16);
	printf("PCI_COMMAND  =%x\n",(int)reg16);

	pci_read_config_word (PCI_BDF(0,0,0), 0x44, &reg16);
	printf("0x44=%x\n",reg16);
#endif

	immap->im_pcix.cfg_addr = 0x8000000d;
	immap->im_pcix.cfg_data = 0x08200080;

	if(get_pci_mode())
	{
		immap->im_pcix.cfg_addr = 0x80000044;
		immap->im_pcix.cfg_data = 0x09000080;
	}


	/*
	 * Clear non-reserved bits in status register.
	 */
	pci_write_config_word(PCI_BDF(0,0,0), PCI_STATUS, 0xffff);
	pci_write_config_byte(PCI_BDF(0,0,0), PCI_LATENCY_TIMER,0x80);

	pcix->potar1   = (CFG_PCI1_MEM_BASE >> 12) & 0x000fffff;
	pcix->potear1  = 0x00000000;
	pcix->powbar1  = (CFG_PCI1_MEM_BASE >> 12) & 0x000fffff;
	pcix->powbear1 = 0x00000000;
	pcix->powar1   = 0x8004401c;	/* 512M MEM space */

	pcix->potar2   = 0x00000000;
	pcix->potear2  = ((CFG_PCI1_MEM_BASE + 0x20000000) >> 12) & 0x000fffff;
	pcix->powbar2  = 0x00000000;
	pcix->powbear2 = ((CFG_PCI1_MEM_BASE + 0x20000000) >> 12) & 0x000fffff;
	pcix->powar2   = 0x8004401c;	/* 512M MEM space */

	pcix->potar3   = 0x00000000;
	pcix->potear3  = 0x00000000;
	pcix->powbar3  = (CFG_PCI1_IO_BASE >> 12) & 0x000fffff;
	pcix->powbear3 = 0x00000000;
	pcix->powar3   = 0x80088017;	/* 16M IO space */

	pcix->pitar1   = (0x4000000 >> 12) & 0x000fffff;
	pcix->piwbar1  = 0x00000000;
	pcix->piwar1   = 0xa0f55019;	/* Enable, Prefetch, Local Mem, Snoop R/W, 64M */

	pcix->pitar2   = (0x8000000 >> 12) & 0x000fffff;
	pcix->piwbar2  = 0x00000000;
	pcix->piwar2   = 0xa0f55019;	/* Enable, Prefetch, Local Mem, Snoop R/W, 64M */

	if(!get_pci_mode())
	{
		/*
		 * Hose scan.
		 */
		printf("PCI bus scan...\n");

		udelay(10000);

		pci_register_hose(hose);

		hose->last_busno = pci_hose_scan(hose);
	}

}
#endif

void test_64bit_pci(void)
{
	uint *p = (uint*)0x88000000;

	/* caching must be enabled for the PCI regions in the TLB */

	for (*p = 0; p < ((uint *)((8 * 1024) | 0x88000000)); p++) {
		if (((unsigned int)p & 0x1f) == 0) {
			ppcDcbz((unsigned long) p);
		}
		*p = (unsigned int)0xdeadbeef;
		if (((unsigned int)p & 0x1c) == 0x1c) {
			ppcDcbf((unsigned long) p);
		}
	}
}
