/*
 * Copyright 2004 Freescale Semiconductor.
 * (C) Copyright 2002,2003 Motorola,Inc.
 * Xianghua Xiao <X.Xiao@motorola.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * ep8560m board configuration file
 *
 * Please refer to doc/README.mpc85xx for more info.
 *
 * Make sure you change the MAC address and other network params first,
 * search for CONFIG_ETHADDR, CONFIG_SERVERIP, etc in this file.
 */



#ifndef __CONFIG_H
#define __CONFIG_H

#undef  DEBUG

/* High Level Configuration Options */
#define CONFIG_BOOKE 		1	/* BOOKE */
#define CONFIG_E500		1	/* BOOKE e500 family */
#define CONFIG_MPC85xx		1	/* MPC8540/MPC8560 */
#define CONFIG_CPM2		1	/* has CPM2 */
#undef CONFIG_MPC8560      	1	/* MPC8560 specific */
#define CONFIG_EP85xxM      		/* EP85xxM board specific */
#undef  CONFIG_EP8560M      		/* EP8560M board specific */
#define CONFIG_EP8555M      		/* EP8555M board specific */
#undef  CONFIG_EP8541M      		/* EP8541M board specific */

#define CONFIG_SDRAM_64
#undef  CONFIG_SDRAM_128
#undef  CONFIG_SDRAM_256

#define CONFIG_FLASH_16
#undef  CONFIG_FLASH_32

#define CONFIG_PCI
#define CONFIG_TSEC_ENET 		/* tsec ethernet support */
#define CONFIG_ENV_OVERWRITE
#undef	CONFIG_DDR_ECC_DISCRETE	/* enable ECC for onboard, discrete DDR */
#define CONFIG_DDR_DLL			/* possible DLL fix needed */
#define CONFIG_DDR_2T_TIMING	/* Sets the 2T timing bit */
#define CONFIG_PIC				/* Enables external interrupts */
#define CONFIG_CLK_OUT			/* Enable CLK_OUT signal for testing */

#define CONFIG_BOARD_EARLY_INIT_F	1	/* Call board_pre_init */
#define CONFIG_BOARD_EARLY_INIT_R	1	/* Call board_pre_init */

/* BCSR Registers */
#define BCSR2_CPU_TYPE 		0xf8
#define BCSR2_RESERVED 		0x04
#define BCSR2_DDR_SZ		0x03

#define BCSR9_M66EN 		0x80
#define BCSR9_MONARCH 		0x40
#define BCSR9_PCIXCAP		0x30
#define BCSR9_PTENB			0x08
#define BCSR9_PTID			0x07

#define BCSR11_FLASH_RDY 	0x80
#define BCSR11_FLASH_WP 	0x40
#define BCSR11_RESERVED 	0x30
#define BCSR11_SRAM_EN	 	0x08
#define BCSR11_SRAM_WP		0x04
#define BCSR11_STTM_ADDR	0x03

#define BCSR13_GETH1_RESET 	0x80
#define BCSR13_GETH1_COMA 	0x40
#define BCSR13_GETH2_RESET 	0x20
#define BCSR13_GETH2_COMA 	0x10
#define BCSR13_RESERVED		0x0f

#define BCSR15_PCI_INTD_MASK 	0x80
#define BCSR15_PCI_INTC_MASK 	0x40
#define BCSR15_PCI_INTB_MASK 	0x20
#define BCSR15_PCI_INTA_MASK 	0x10
#define BCSR15_PCI_INTD_STAT 	0x80
#define BCSR15_PCI_INTC_STAT 	0x40
#define BCSR15_PCI_INTB_STAT 	0x20
#define BCSR15_PCI_INTA_STAT 	0x10


/*
 * sysclk for MPC85xx
 *
 * Valid values are:
 *    33333333
 *    66666667
 *    100000000
 *    133000000
 *
 * Most PCI cards are still 33Mhz, so in the presence of PCI, 33MHz
 * is likely the desired value here, so that is now the default.
 * The board, however, can run at 66MHz.  In any event, this value
 * must match the settings of some switches.
 */

#define SYS_CLK_FREQ_33			33333333
#define SYS_CLK_FREQ_66			66666667
#define SYS_CLK_FREQ_100		100000000
#define SYS_CLK_FREQ_133		133000000

#if 0
#ifndef CONFIG_SYS_CLK_FREQ
#define CONFIG_SYS_CLK_FREQ		SYS_CLK_FREQ_33
#endif
#endif

/*
 * These can be toggled for performance analysis, otherwise use default.
 */
#define CONFIG_L2_CACHE			/* toggle L2 cache */
#undef  CONFIG_BTB				/* toggle branch predition */
#undef  CONFIG_ADDR_STREAMING	/* toggle addr streaming */

#define CFG_INIT_DBCR (DBCR_IDM + 0x01000000)	/* Enable Debug Exceptions */

#undef  CFG_DRAM_TEST			/* memory test, takes time */
#define CFG_MEMTEST_START		0x00200000	/* memtest region */
#define CFG_MEMTEST_END			0x00400000

/*
 * Base addresses -- Note these are effective addresses where the
 * actual resources get mapped (not physical addresses)
 */
#define CFG_CCSRBAR_DEFAULT		0xff700000	/* CCSRBAR Default */
#define CFG_CCSRBAR				0xe0000000	/* relocated CCSRBAR */
#define CFG_IMMR				CFG_CCSRBAR	/* PQII uses CFG_IMMR */

/*
 * DDR Setup
 */
#define CFG_DDR_SDRAM_BASE		0x00000000	/* DDR is system memory*/
#define CFG_SDRAM_BASE			CFG_DDR_SDRAM_BASE

/*
 * Manually set up DDR parameters
 */
#ifdef CONFIG_SDRAM_64
#define CFG_SDRAM_SIZE			64			/* DDR is 64MB */
#define CFG_DDR_CS0_BNDS		0x00000003	/* 0-64MB */
#define CFG_DDR_CS0_CONFIG		0x80000001  /* 12 rows, 9 columns */
#define CFG_DDR_TIMING_1		0x37334342
#define CFG_DDR_TIMING_2		0x00000800	/* P9-45,may need tuning */
#define CFG_DDR_CONTROL			0xC2000000	/* unbuffered,no DYN_PWR */
#define CFG_DDR_MODE			0x00000062	/* DLL,normal,seq,4/2.5 */
#define CFG_DDR_INTERVAL		0x045b0100	/* autocharge,no open page */
#endif /* CONFIG_SDRAM_64 */

#ifdef CONFIG_SDRAM_128
#define CFG_SDRAM_SIZE			128			/* DDR is 128MB */
#define CFG_DDR_CS0_BNDS		0x00000007	/* 0-128MB */
#define CFG_DDR_CS0_CONFIG		0x80000101  /* 13 rows, 9 columns */
#define CFG_DDR_TIMING_1		0x37334342
#define CFG_DDR_TIMING_2		0x00000800	/* P9-45,may need tuning */
#define CFG_DDR_CONTROL			0xC2000000	/* unbuffered,no DYN_PWR */
#define CFG_DDR_MODE			0x00000062	/* DLL,normal,seq,4/2.5 */
#define CFG_DDR_INTERVAL		0x045b0100	/* autocharge,no open page */
#endif /* CONFIG_SDRAM_128 */

#ifdef CONFIG_SDRAM_256
#define CFG_SDRAM_SIZE			256			/* DDR is 256MB */
#define CFG_DDR_CS0_BNDS		0x0000000f	/* 0-256MB */
#define CFG_DDR_CS0_CONFIG		0x80000102  /* 13 rows, 10 columns */
#define CFG_DDR_TIMING_1		0x37334342
#define CFG_DDR_TIMING_2		0x00000800	/* P9-45,may need tuning */
#define CFG_DDR_CONTROL			0xC2000000	/* unbuffered,no DYN_PWR */
#define CFG_DDR_MODE			0x00000062	/* DLL,normal,seq,4/2.5 */
#define CFG_DDR_INTERVAL		0x045b0100	/* autocharge,no open page */
#endif /* CONFIG_SDRAM_256 */

/*
 * Local Bus
 */
#define CFG_LBC_BASE			0xe0000000	/* Localbus */

#define CFG_FLASH_MAP_BASE		0xf0000000  /* 256M mapped */
#ifdef CONFIG_FLASH_16
#define CFG_FLASH_BASE			0xff000000	/* start of FLASH 32M */
#define CFG_BR0_PRELIM			0xff001801	/* port size 32bit */
#define CFG_OR0_PRELIM			0xff000ef1	/* 16MB Flash */
#define CFG_MAX_FLASH_SECT		128			/* sectors per device */
#else
#define CFG_FLASH_BASE			0xfe000000	/* start of FLASH 32M */
#define CFG_BR0_PRELIM			0xfe001801	/* port size 32bit */
#define CFG_OR0_PRELIM			0xfe000ef1	/* 32MB Flash */
#define CFG_MAX_FLASH_SECT		128			/* sectors per device */
#endif
#define CFG_MAX_FLASH_BANKS		1			/* number of banks */

#undef	CFG_FLASH_CHECKSUM
#define CFG_FLASH_ERASE_TOUT	60000		/* Flash Erase Timeout (ms) */
#define CFG_FLASH_WRITE_TOUT	500			/* Flash Write Timeout (ms) */

#define CFG_MONITOR_BASE    	TEXT_BASE	/* start of monitor */

#if (CFG_MONITOR_BASE < CFG_FLASH_BASE)
#define CFG_RAMBOOT
#else
#undef  CFG_RAMBOOT
#endif

#define CFG_FLASH_CFI_DRIVER
#define CFG_FLASH_CFI
#define CFG_FLASH_EMPTY_INFO

#undef CONFIG_CLOCKS_IN_MHZ

/*
 * Local Bus Definitions
 */
#define  CFG_LBC_LCRR		0x00000004	/* LB clock ratio reg */

/*
 * 2MB, 16-bit wide ----- FIX ME ------
 */
#define CFG_BR2_PRELIM		0xef001001
#define CFG_OR2_PRELIM		0xffe00ef0
#define CFG_BCSR			(CFG_BR2_PRELIM & 0xffff8000)
#define CFG_NVRAM			(CFG_BCSR & 0x100000)

#define CONFIG_L1_INIT_RAM
#define CFG_INIT_RAM_LOCK 	1
#define CFG_INIT_RAM_ADDR	0xe4010000		/* Initial RAM address */
#define CFG_INIT_RAM_END   	0x4000	    	/* End of used area in RAM */
#define CFG_L2_SRAM_BASE	0x20000000

#define CFG_GBL_DATA_SIZE  	128		/* num bytes initial data */
#define CFG_GBL_DATA_OFFSET	(CFG_INIT_RAM_END - CFG_GBL_DATA_SIZE)
#define CFG_INIT_SP_OFFSET	CFG_GBL_DATA_OFFSET

#define CFG_MONITOR_LEN		(256 * 1024)    /* Reserve 256 kB for Mon */
#define CFG_MALLOC_LEN		(128 * 1024)    /* Reserved for malloc */

/* Serial Port */
#define CONFIG_BAUDRATE	 	115200
#define CFG_BAUDRATE_TABLE  \
	{300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200}
#define CONFIG_CONS_INDEX       1  /* which serial channel for console */

#if defined(CONFIG_EP8560M) || defined (CONFIG_EP8555M)
#define CONFIG_CONS_ON_SCC	/* define if console on SCC */
#undef  CONFIG_CONS_NONE	/* define if console on something else */
#endif

#if defined (CONFIG_EP8541M)
#undef	CONFIG_SERIAL_SOFTWARE_FIFO
#undef  CONFIG_CONS_NONE	/* define if console on something else */
#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE    1
#define CFG_NS16550_CLK		get_bus_freq(0)

#define CFG_NS16550_COM1        (CFG_CCSRBAR+0x4500)
#define CFG_NS16550_COM2        (CFG_CCSRBAR+0x4600)
#endif

/* Use the HUSH parser */
#define CFG_HUSH_PARSER
#ifdef  CFG_HUSH_PARSER
#define CFG_PROMPT_HUSH_PS2 "> "
#endif

/* I2C */
#define  CONFIG_HARD_I2C			/* I2C with hardware support*/
#undef	CONFIG_SOFT_I2C				/* I2C bit-banged */
#define CFG_I2C_SPEED		400000	/* I2C speed and slave address */
#define CFG_I2C_SLAVE		0x7F
#define CFG_I2C_NOPROBES	{0x69}	/* Don't probe these addrs */
#define CFG_I2C_EEPROM_ADDR_LEN 1

/*
 * General PCI
 * Addresses are mapped 1-1.
 */
#define CFG_PCI1_MEM_BASE	0x80000000
#define CFG_PCI1_MEM_PHYS	CFG_PCI1_MEM_BASE
#define CFG_PCI1_MEM_SIZE	0x40000000	/* 1G */
#define CFG_PCI1_IO_BASE	0xC0000000
#define CFG_PCI1_IO_PHYS	CFG_PCI1_IO_BASE
#define CFG_PCI1_IO_SIZE	0x1000000	/* 16M */

#if defined(CONFIG_PCI)

#define CONFIG_NET_MULTI
#define	CONFIG_PCI_PNP	               	/* do pci plug-and-play */

#define CONFIG_EEPRO100
#undef  CONFIG_TULIP

#if !defined(CONFIG_PCI_PNP)
    #define PCI_ENET0_IOADDR	0xc0000000
    #define PCI_ENET0_MEMADDR	0x80000000
    #define PCI_IDSEL_NUMBER	0x0c 	/* slot0->3(IDSEL)=12->15 */
#endif

#define CONFIG_PCI_SCAN_SHOW			/* show pci devices on startup */
#define CFG_PCI_SUBSYS_VENDORID 0x1057  /* Freescale */

#endif	/* CONFIG_PCI */


#if defined(CONFIG_TSEC_ENET)

#ifndef CONFIG_NET_MULTI
#define CONFIG_NET_MULTI 	1
#endif

#define CONFIG_MII					/* MII PHY management */
#define CONFIG_MPC85XX_TSEC1_REDUCED	1
#define CONFIG_MPC85XX_TSEC1_NAME	"TSEC0"
#define CONFIG_MPC85XX_TSEC2_REDUCED	1
#define CONFIG_MPC85XX_TSEC2_NAME	"TSEC1"
#define TSEC1_PHY_ADDR			0
#define TSEC2_PHY_ADDR			1
#define TSEC1_PHYIDX			0
#define TSEC2_PHYIDX			0

/* Options are: TSEC[0-1] */
#define CONFIG_ETHPRIME		"TSEC0"

#endif /* CONFIG_TSEC_ENET */


/*
 * Environment
 */
#ifndef CFG_RAMBOOT
  #define CFG_ENV_IS_IN_FLASH	1
  #define CFG_ENV_ADDR		(CFG_MONITOR_BASE + 0x40000)
  #define CFG_ENV_SECT_SIZE	0x40000	/* 256K(one sector) for env */
  #define CFG_ENV_SIZE		0x2000
#else
  #define CFG_NO_FLASH		1	/* Flash is not usable now */
  #define CFG_ENV_IS_NOWHERE	1	/* Store ENV in memory only */
  #define CFG_ENV_ADDR		(CFG_MONITOR_BASE - 0x1000)
  #define CFG_ENV_SIZE		0x2000
#endif

#define CONFIG_LOADS_ECHO	1	/* echo on for serial download */
#define CFG_LOADS_BAUD_CHANGE	1	/* allow baudrate change */

#if defined(CFG_RAMBOOT)
  #if defined(CONFIG_PCI)
    #define  CONFIG_COMMANDS	((CONFIG_CMD_DFL	\
				| CFG_CMD_PCI		\
				| CFG_CMD_I2C)		\
				& ~(CFG_CMD_ENV		\
				| CFG_CMD_LOADS))
  #elif defined(CONFIG_TSEC_ENET)
    #define  CONFIG_COMMANDS	((CONFIG_CMD_DFL	\
				| CFG_CMD_PING		\
				| CFG_CMD_I2C)		\
				& ~(CFG_CMD_ENV		\
				| CFG_CMD_LOADS))
  #endif
#else
  #if defined(CONFIG_TSEC_ENET) && defined(CONFIG_PCI)
    #define  CONFIG_COMMANDS	(CONFIG_CMD_DFL		\
				| CFG_CMD_PCI		\
				| CFG_CMD_MII		\
				| CFG_CMD_PING		\
				| CFG_POST_ETHER	\
				| CFG_CMD_I2C)
  #elif defined(CONFIG_PCI) && !defined(CONFIG_TSEC_ENET)
    #define  CONFIG_COMMANDS	(CONFIG_CMD_DFL		\
				| CFG_CMD_PCI		\
				| CFG_CMD_PING		\
				| CFG_CMD_I2C)
  #elif defined(CONFIG_TSEC_ENET) && !defined(CONFIG_PCI)
    #define  CONFIG_COMMANDS	(CONFIG_CMD_DFL		\
				| CFG_CMD_MII		\
				| CFG_CMD_PING		\
				| CFG_CMD_I2C)
  #else
    #define  CONFIG_COMMANDS	(CFG_CMD_BDI		\
				| CFG_CMD_CACHE		\
				| CFG_CMD_FLASH		\
				| CFG_CMD_MEMORY		\
				| CFG_CMD_ENV		\ 
				| CFG_CMD_IRQ		\
				| CFG_CMD_CONSOLE		\
				| CFG_CMD_EEPROM		\
				| CFG_CMD_ASKENV		\
				| CFG_CMD_ECHO		\
				| CFG_CMD_I2C		\
				| CFG_CMD_REGINFO		\
				| CFG_CMD_IMMAP		\
				| CFG_CMD_BSP		\
				| CFG_CMD_DIAG)
  #endif
#endif

#include <cmd_confdefs.h>

#undef CONFIG_WATCHDOG			/* watchdog disabled */

/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP					/* undef to save memory	*/
#define CFG_LOAD_ADDR		0x1000000	/* default load address */
#define CFG_PROMPT	"ep=> "				/* Monitor Command Prompt */

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
    #define CFG_CBSIZE		1024	/* Console I/O Buffer Size */
#else
    #define CFG_CBSIZE		256		/* Console I/O Buffer Size */
#endif

#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS		16			/* max number of command args */
#define CFG_BARGSIZE	CFG_CBSIZE	/* Boot Argument Buffer Size */
#define CFG_HZ			1000		/* decrementer freq: 1ms ticks */

/*
 * For booting Linux, the board info and command line data
 * have to be in the first 8 MB of memory, since this is
 * the maximum mapped by the Linux kernel during initialization.
 */
#define CFG_BOOTMAPSZ	(8 << 20)	/* Initial Memory map for Linux*/

/* Cache Configuration */
#define CFG_DCACHE_SIZE		32768
#define CFG_CACHELINE_SIZE	32
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CACHELINE_SHIFT	5	/*log base 2 of the above value*/
#endif

/*
 * Internal Definitions
 *
 * Boot Flags
 */
#define BOOTFLAG_COLD	0x01		/* Normal Power-On: Boot from FLASH */
#define BOOTFLAG_WARM	0x02		/* Software reboot */

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE	230400	/* speed to run kgdb serial port */
#define CONFIG_KGDB_SER_INDEX	2	/* which serial port to use */
#endif

/*
 * Environment Configuration
 */

/* The mac addresses for all ethernet interface */
#if defined(CONFIG_TSEC_ENET)
#define CONFIG_ETHADDR   00:10:EC:00:00:FD
#define CONFIG_HAS_ETH1
#define CONFIG_ETH1ADDR  00:10:EC:80:00:FD
#endif

#define CONFIG_IPADDR    192.168.1.253

#define CONFIG_HOSTNAME		unknown
#define CONFIG_ROOTPATH		/nfsroot
#define CONFIG_BOOTFILE		your.uImage

#define CONFIG_SERVERIP  192.168.1.1
#define CONFIG_GATEWAYIP 192.168.1.1
#define CONFIG_NETMASK   255.255.255.0

#define CONFIG_LOADADDR  200000	/* default location for tftp and bootm */

#define CONFIG_BOOTDELAY -1	/* -1 disables auto-boot */
#undef  CONFIG_BOOTARGS		/* the boot command will set bootargs */

#define	CONFIG_EXTRA_ENV_SETTINGS				        \
   "netdev=eth0\0"                                                      \
   "consoledev=ttyS0\0"                                                 \
   "ramdiskaddr=400000\0"						\
   "ramdiskfile=your.ramdisk.u-boot\0"

#define CONFIG_NFSBOOTCOMMAND	                                        \
   "setenv bootargs root=/dev/nfs rw "                                  \
      "nfsroot=$serverip:$rootpath "                                    \
      "ip=$ipaddr:$serverip:$gatewayip:$netmask:$hostname:$netdev:off " \
      "console=$consoledev,$baudrate $othbootargs;"                     \
   "tftp $loadaddr $bootfile;"                                          \
   "bootm $loadaddr"

#define CONFIG_RAMBOOTCOMMAND \
   "setenv bootargs root=/dev/ram rw "                                  \
      "console=$consoledev,$baudrate $othbootargs;"                     \
   "tftp $ramdiskaddr $ramdiskfile;"                                    \
   "tftp $loadaddr $bootfile;"                                          \
   "bootm $loadaddr $ramdiskaddr"

#define CONFIG_BOOTCOMMAND  CONFIG_NFSBOOTCOMMAND


/* External Interrupts */
#define MAX_NUM_EXT_INTS	45
#define CTPR_TASK_PRIORITY	9

#define EIVPR_IRQ0_VECTOR	0x0100
#define EIVPR_IRQ1_VECTOR	0x0200
#define EIVPR_IRQ2_VECTOR	0x0300
#define EIVPR_IRQ3_VECTOR	0x0400
#define EIVPR_IRQ4_VECTOR	0x0500
#define EIVPR_IRQ5_VECTOR	0x0600
#define EIVPR_IRQ6_VECTOR	0x0700
#define EIVPR_IRQ7_VECTOR	0x0800
#define EIVPR_IRQ8_VECTOR	0x0900

#define EIVPR_IRQ0_PRIORITY		10
#define EIVPR_IRQ1_PRIORITY		10
#define EIVPR_IRQ2_PRIORITY		10
#define EIVPR_IRQ3_PRIORITY		10
#define EIVPR_IRQ4_PRIORITY		10
#define EIVPR_IRQ5_PRIORITY		10
#define EIVPR_IRQ6_PRIORITY		10
#define EIVPR_IRQ7_PRIORITY		10
#define EIVPR_IRQ8_PRIORITY		10

#define EIVPR_LOW_NEG_EDGE		(0<<23) /* Polarity is active-low or negative edge triggered */
#define EIVPR_HI_POS_EDGE		(1<<23) /* Polarity is active-high or positive edge-triggered */

#define EIVPR_IRQ0_POLARITY		EIVPR_LOW_NEG_EDGE
#define EIVPR_IRQ1_POLARITY		EIVPR_LOW_NEG_EDGE
#define EIVPR_IRQ2_POLARITY		EIVPR_LOW_NEG_EDGE
#define EIVPR_IRQ3_POLARITY		EIVPR_LOW_NEG_EDGE
#define EIVPR_IRQ4_POLARITY		EIVPR_LOW_NEG_EDGE
#define EIVPR_IRQ5_POLARITY		EIVPR_LOW_NEG_EDGE
#define EIVPR_IRQ6_POLARITY		EIVPR_LOW_NEG_EDGE
#define EIVPR_IRQ7_POLARITY		EIVPR_LOW_NEG_EDGE
#define EIVPR_IRQ8_POLARITY		EIVPR_LOW_NEG_EDGE

#undef  EIVPR_IRQ0
#define EIVPR_IRQ1
#define EIVPR_IRQ2
#define EIVPR_IRQ3
#define EIVPR_IRQ4
#define EIVPR_IRQ5
#define EIVPR_IRQ6
#undef  EIVPR_IRQ7
#undef  EIVPR_IRQ8

#define EIVPR_MSK			0x80000000
#define EIVPR_A				0x40000000
#define EIVPR_P				0x00800000
#define EIVPR_S				0x00400000
#define EIVPR_PRIORITY		0x000F0000
#define EIVPR_VECTOR		0x0000FFFF


#endif	/* __CONFIG_H */
